require 'helper'

describe DistributedCache do
  context 'config' do
    it "should be able to configure itself" do
      configure_test_distributed_cache

      DistributedCache.config.file_servers.should == ['localhost']
      DistributedCache.config.cache_dir = "#{BASE_TMP_DIR}/cache"
      DistributedCache.config.bundle_dir = "#{BASE_TMP_DIR}/bundles"
    end
  end

  context "update_remote_cache" do
    before(:each) do
      configure_test_distributed_cache
    end

    it "should create the full cache tar file" do
      DistributedCache.update_remote_cache CachableKlass
      DistributedCache::Manifest.new(DistributedCache.config, CachableKlass.cache_name, CachableKlass.cache_version).tap do |manifest|
        manifest.cache_name.should == CachableKlass.cache_name
        manifest.cache_version.should == CachableKlass.cache_version
        manifest.files.size.should == 1
        manifest.files[0][0].should == "#{CachableKlass.cache_name}/#{Time.now.strftime('%Y%m%d')}/#{CachableKlass.cache_version}/test-1.tgz"
      end
    end

    it "should add the partial data as another tar file" do
      DistributedCache.update_remote_cache CachableKlass
      DistributedCache.update_remote_cache CachableKlass
      DistributedCache::Manifest.new(DistributedCache.config, CachableKlass.cache_name, CachableKlass.cache_version).tap do |manifest|
        manifest.cache_name.should == CachableKlass.cache_name
        manifest.cache_version.should == CachableKlass.cache_version
        manifest.files.size.should == 2
        manifest.files[0][0].should == "#{CachableKlass.cache_name}/#{Time.now.strftime('%Y%m%d')}/#{CachableKlass.cache_version}/test-1.tgz"
        manifest.files[1][0].should == "#{CachableKlass.cache_name}/#{Time.now.strftime('%Y%m%d')}/#{CachableKlass.cache_version}/test-2.tar"
      end
    end
  end

  context "update_local_cache" do
    def update_remote_cache_for_context_update_local_cache
      old_bundle_dir = DistributedCache.config.bundle_dir
      old_cache_dir = DistributedCache.config.cache_dir

      DistributedCache.configure do |config|
        config.bundle_dir = "#{BASE_TMP_DIR}/remote/cache/bundles"
        config.cache_dir = "#{BASE_TMP_DIR}/remote/cache/cache"
      end

      DistributedCache.update_remote_cache CachableKlass

      DistributedCache.configure do |config|
        config.bundle_dir = old_bundle_dir
        config.cache_dir = old_cache_dir
      end
    end

    before(:each) do
      configure_test_distributed_cache
      update_remote_cache_for_context_update_local_cache
    end

    it "should be able to install the full tar" do
      cache_path = "#{CachableKlass.cache_name}/#{Time.now.strftime('%Y%m%d')}/#{CachableKlass.cache_version}"
      manifest_file = "#{BASE_TMP_DIR}/remote/cache/bundles/test/latest/manifest.yml"
      tar_file_1 = "#{BASE_TMP_DIR}/remote/cache/bundles/#{cache_path}/test-1.tgz"

      File.exists?(manifest_file).should be_true
      File.exists?(tar_file_1).should be_true
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/test/latest/manifest.yml", :body => manifest_file
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/#{cache_path}/test-1.tgz", :body => tar_file_1
      DistributedCache.update_local_cache CachableKlass.cache_name

      [['full1', 'test1'], ['full2', 'test2'], ['full3', 'test3']].each do |name, data|
        file = "#{DistributedCache.config.cache_dir}/#{CachableKlass.cache_name}/#{name}"
        File.exists?(file).should be_true
        File.read(file).should == data
      end
    end

    it "should use the partial tar override existing cache files" do
      cache_path = "#{CachableKlass.cache_name}/#{Time.now.strftime('%Y%m%d')}/#{CachableKlass.cache_version}"
      manifest_file = "#{BASE_TMP_DIR}/remote/cache/bundles/test/latest/manifest.yml"
      tar_file_1 = "#{BASE_TMP_DIR}/remote/cache/bundles/#{cache_path}/test-1.tgz"
      tar_file_2 = "#{BASE_TMP_DIR}/remote/cache/bundles/#{cache_path}/test-2.tar"

      File.exists?(manifest_file).should be_true
      File.exists?(tar_file_1).should be_true
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/test/latest/manifest.yml", :body => manifest_file
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/#{cache_path}/test-1.tgz", :body => tar_file_1

      DistributedCache.update_local_cache CachableKlass.cache_name

      [['full1', 'test1'], ['full2', 'test2'], ['full3', 'test3']].each do |name, data|
        file = "#{DistributedCache.config.cache_dir}/#{CachableKlass.cache_name}/#{name}"
        File.exists?(file).should be_true
        File.read(file).should == data
      end

      update_remote_cache_for_context_update_local_cache

      FakeWeb.clean_registry
      File.exists?(tar_file_2).should be_true
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/test/latest/manifest.yml", :body => manifest_file
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/#{cache_path}/test-2.tar", :body => tar_file_2

      DistributedCache.update_local_cache CachableKlass.cache_name

      [['full1', 'test1'], ['full2', 'changed2'], ['full3', 'test3'], ['partial1', '1'], ['partial2', '2']].each do |name, data|
        file = "#{DistributedCache.config.cache_dir}/#{CachableKlass.cache_name}/#{name}"
        File.exists?(file).should be_true
        File.read(file).should == data
      end
    end

    it "should download the tar file if it is missing locally and untar all the files after it" do
      update_remote_cache_for_context_update_local_cache

      cache_path = "#{CachableKlass.cache_name}/#{Time.now.strftime('%Y%m%d')}/#{CachableKlass.cache_version}"
      manifest_file = "#{BASE_TMP_DIR}/remote/cache/bundles/test/latest/manifest.yml"
      tar_file_1 = "#{BASE_TMP_DIR}/remote/cache/bundles/#{cache_path}/test-1.tgz"
      tar_file_2 = "#{BASE_TMP_DIR}/remote/cache/bundles/#{cache_path}/test-2.tar"

      File.exists?(manifest_file).should be_true
      File.exists?(tar_file_1).should be_true
      File.exists?(tar_file_2).should be_true
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/test/latest/manifest.yml", :body => manifest_file
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/#{cache_path}/test-1.tgz", :body => tar_file_1
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/#{cache_path}/test-2.tar", :body => tar_file_2

      DistributedCache.update_local_cache CachableKlass.cache_name

      [['full1', 'test1'], ['full2', 'changed2'], ['full3', 'test3'], ['partial1', '1'], ['partial2', '2']].each do |name, data|
        file = "#{DistributedCache.config.cache_dir}/#{CachableKlass.cache_name}/#{name}"
        File.exists?(file).should be_true
        File.read(file).should == data
      end

      local_tar_file_1 = "#{DistributedCache.config.bundle_dir}/#{cache_path}/test-1.tgz"
      File.exists?(local_tar_file_1).should be_true
      File.unlink local_tar_file_1

      FakeWeb.clean_registry
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/test/latest/manifest.yml", :body => manifest_file
      FakeWeb.register_uri :get, "http://localhost/cache/bundles/#{cache_path}/test-1.tgz", :body => tar_file_1

      DistributedCache.update_local_cache CachableKlass.cache_name

      [['full1', 'test1'], ['full2', 'changed2'], ['full3', 'test3'], ['partial1', '1'], ['partial2', '2']].each do |name, data|
        file = "#{DistributedCache.config.cache_dir}/#{CachableKlass.cache_name}/#{name}"
        File.exists?(file).should be_true
        File.read(file).should == data
      end
    end
  end
end
